#!/usr/bin/env python
# -*- coding: utf-8 -*-


# author: Pocholo
# date: 9/5/20

# Comments directed to http://gimpchat.com

# Installation:
# This script should be placed in the user plugins folder

# Copyright (C)

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by   
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY# without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <https://www.gnu.org/licenses/>.   


from gimpfu import *
import gettext
import os, sys,time 
gettext.install("gimp20", gimp.locale_directory, unicode=True)

def pm_halloween_graveyard_scene(customtext, fontname, hazeglowColor, textColor):

    #Initiates the temporary state
    pdb.gimp_context_push()
    pdb.gimp_context_set_defaults()
			
	#Create starry background 
    img = pdb.gimp_image_new(1400, 850, RGB)
    backLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Background", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_insert_layer(img, backLayer, None, 0)
    pdb.gimp_context_set_foreground("Black")
    pdb.gimp_drawable_fill(backLayer, FILL_FOREGROUND)
    pdb.plug_in_hsv_noise(img, backLayer, 8, 3, 5, 255)
    pdb.plug_in_gauss(img, backLayer, 1, 1, 1)
    pdb.plug_in_sparkle(img, backLayer, 0.004, 1.00, 4, 4, -1, 1.00, 0, 0, 0, FALSE, FALSE, FALSE, 0)
    
    #Haze
    hazeLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Haze", 10, LAYER_MODE_NORMAL_LEGACY)
    pdb.gimp_image_insert_layer(img, hazeLayer, None, 0)
    pdb.gimp_context_set_foreground("Black")
    pdb.gimp_drawable_fill(hazeLayer, FILL_FOREGROUND)
    pdb.gimp_context_set_foreground("#000000")
    pdb.gimp_context_set_background(hazeglowColor)
    pdb.gimp_edit_blend(hazeLayer, 
                                0,   #blend_mode - fg to bg
                                0,   #paint_mode - normal
                                0,   #gradient_type - linear
                              100,   #opacity, 
                                0,   #offset
                                0,   #repeat
                            FALSE,   #reverse
                            FALSE,   #supersample
                                0,   #max_depth
                                0,   #threshold
                             TRUE,   #dither
                    img.width / 2,   #x1
                 0,   #y1
                    img.width / 2,   #x2
                       img.height)   #y2 
                      
    #Image process
    pdb.gimp_display_new(img)
    
    #Create the Moon
    moonLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Moon", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, moonLayer, 0)
    pdb.gimp_drawable_fill(moonLayer, FILL_TRANSPARENT)
    pdb.gimp_context_set_foreground("White")
    pdb.gimp_image_select_ellipse(img, 2, 375, 60, 650, 650)
    pdb.gimp_edit_bucket_fill(moonLayer, BUCKET_FILL_FG, LAYER_MODE_NORMAL, 100, 255, FALSE, 0, 0)
    pdb.plug_in_solid_noise(img, moonLayer, FALSE, FALSE, 0, 15, 0.8, 0.8)
    pdb.gimp_selection_none(img)
    pdb.plug_in_emboss(img, moonLayer, 220, 70, 14, FALSE)
    pdb.gimp_image_select_item(img, 2, moonLayer)
    pdb.gimp_selection_shrink(img, 6)
    pdb.gimp_selection_invert(img)
    pdb.gimp_drawable_edit_clear(moonLayer)
    pdb.gimp_selection_invert(img)
    
    #Moon glow
    brightLayer = pdb.gimp_layer_new_from_drawable(moonLayer, img)
    pdb.gimp_image_insert_layer(img, brightLayer, None, 0)
    pdb.gimp_layer_set_mode(brightLayer, LAYER_MODE_LINEAR_LIGHT)
    pdb.gimp_layer_set_opacity(brightLayer, 40)
    pdb.gimp_context_set_foreground("White")
    pdb.gimp_edit_bucket_fill(brightLayer, BUCKET_FILL_FG, LAYER_MODE_NORMAL, 100, 255, FALSE, 0, 0)
    glowLayer = pdb.gimp_layer_new_from_drawable(moonLayer, img)
    pdb.gimp_image_insert_layer(img, glowLayer, None, 2)
    pdb.gimp_layer_set_mode(glowLayer, LAYER_MODE_NORMAL_LEGACY)
    pdb.gimp_context_set_foreground(hazeglowColor)
    pdb.gimp_edit_bucket_fill(glowLayer, BUCKET_FILL_FG, LAYER_MODE_NORMAL, 100, 255, FALSE, 0, 0)
    pdb.gimp_layer_set_opacity(glowLayer, 40)
    pdb.gimp_selection_none(img)
    pdb.plug_in_gauss(img, glowLayer, 150, 150, 1)
    
    #Create the grassy ground
    groundLayer1 = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Ground #1", 100, LAYER_MODE_NORMAL_LEGACY)
    pdb.gimp_image_insert_layer(img, groundLayer1, None, 0)
    pdb.gimp_image_select_ellipse(img, CHANNEL_OP_REPLACE, -545, 565, 2444, 664)
    pdb.plug_in_sel2path(img, groundLayer1)
    pdb.gimp_context_set_foreground("#000000")
    pdb.gimp_edit_bucket_fill(groundLayer1,  
                            BUCKET_FILL_FG,  #fill_mode
                         LAYER_MODE_NORMAL,  #paint_mode
                                       100,  #opacity
                                       255,  #threshold
                                     FALSE,  #sample_merged
                                         0,  #x
                                         0)  #y
   
    #Grass hill
    pdb.gimp_selection_none(img)
    pdb.gimp_brushes_set_brush("2. Block 03")
    pdb.gimp_context_set_brush_size(100)
    pdb.gimp_context_set_brush_angle(0)
    pdb.gimp_context_set_brush_aspect_ratio(20.00)
    pdb.gimp_context_set_brush_spacing(0.45)
    pdb.gimp_context_set_brush_default_hardness()
    pdb.gimp_context_set_brush_force(1)
    pdb.gimp_context_set_dynamics("Random Size/Angle")  
    pdb.gimp_context_set_stroke_method(STROKE_PAINT_METHOD)
    vectors = pdb.gimp_image_get_vectors_by_name(img, _("Selection"))   
    pdb.gimp_drawable_edit_stroke_item(groundLayer1, vectors)
    pdb.gimp_drawable_edit_stroke_item(groundLayer1, vectors)
    pdb.gimp_drawable_edit_stroke_item(groundLayer1, vectors)
    pdb.gimp_context_set_dynamics("Dynamics Off")
    
    #Fog
    fogLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Fog", 10, LAYER_MODE_NORMAL_LEGACY)
    pdb.gimp_image_insert_layer(img, fogLayer, None, 1)
    pdb.gimp_image_select_rectangle(img, 2, 0, 526, 1400, 325)
    pdb.gimp_selection_feather(img, 100)
    pdb.plug_in_solid_noise(img, fogLayer, FALSE, FALSE, 0, 15, 1.5, 4.0)
    pdb.gimp_selection_none(img)
    pdb.plug_in_gauss(img, fogLayer, 12, 12, 1)
    
    fogLayer2 = pdb.gimp_layer_new_from_drawable(fogLayer, img)
    pdb.gimp_image_insert_layer(img, fogLayer2, None, 0)
    pdb.gimp_layer_set_mode(fogLayer2, LAYER_MODE_NORMAL_LEGACY)
    pdb.gimp_layer_set_opacity(fogLayer2, 10)
    
    #Tombstones
    #Tombstones #1
    ttLayer = pdb.gimp_layer_new(img, 200, 450, RGBA_IMAGE, "Tombstones", 100, LAYER_MODE_NORMAL_LEGACY)
    pdb.gimp_image_insert_layer(img, ttLayer, None, 2)
    pdb.gimp_image_select_round_rectangle(img, 2, 0, 175, 200, 450, 85, 85)
    pdb.gimp_context_set_foreground("#000000")
    pdb.gimp_edit_bucket_fill(ttLayer,  
                       BUCKET_FILL_FG,  #fill_mode
                    LAYER_MODE_NORMAL,  #paint_mode
                                  100,  #opacity
                                  255,  #threshold
                                FALSE,  #sample_merged
                                    0,  #x
                                    0)  #y
    
    pdb.gimp_selection_none(img)
    pdb.gimp_brushes_set_brush("2. Block 01")
    pdb.gimp_context_set_brush_size(200)
    pdb.gimp_context_set_brush_angle(90)
    pdb.gimp_context_set_brush_aspect_ratio(6.00)
    pdb.gimp_context_set_brush_spacing(0.10)
    pdb.gimp_context_set_brush_default_hardness()
    pdb.gimp_context_set_brush_force(1)
    pdb.gimp_paintbrush_default(ttLayer, 2, [102, 110])
    pdb.gimp_context_set_brush_size(150)
    pdb.gimp_context_set_brush_angle(0)
    pdb.gimp_context_set_brush_aspect_ratio(5.00)
    pdb.gimp_paintbrush_default(ttLayer, 2, [102, 69])
    
    #Tombstones #2
    ttLayer2 = pdb.gimp_layer_new_from_drawable(ttLayer, img)
    pdb.gimp_image_insert_layer(img, ttLayer2, None, 2)
    pdb.gimp_layer_set_mode(ttLayer2, LAYER_MODE_NORMAL_LEGACY)
    pdb.gimp_layer_set_opacity(ttLayer2, 100)
    
    #Move tombstones to center and rotate a bit
    pdb.gimp_layer_resize_to_image_size(ttLayer)
    pdb.gimp_drawable_offset(ttLayer, FALSE, OFFSET_TRANSPARENT, 450, 150)
    ttLayer = pdb.gimp_item_transform_rotate(ttLayer, -0.17, TRUE, 700, 425)
    pdb.gimp_layer_resize_to_image_size(ttLayer)
    pdb.gimp_layer_scale(ttLayer2, 156, 350, TRUE)
    pdb.gimp_layer_resize_to_image_size(ttLayer2)
    pdb.gimp_drawable_offset(ttLayer2, FALSE, OFFSET_TRANSPARENT, 725, 200)
    ttLayer2 = pdb.gimp_item_transform_rotate(ttLayer2, 0.17, TRUE, 700, 425)
    pdb.gimp_layer_resize_to_image_size(ttLayer2)
    
    
    #Bats
    batsLayer = pdb.gimp_layer_new(img, 300, 260, RGBA_IMAGE, "Bats", 100, LAYER_MODE_NORMAL_LEGACY)
    pdb.gimp_image_insert_layer(img, batsLayer, None, 0)
    pdb.gimp_drawable_fill(batsLayer, FILL_TRANSPARENT)
    pdb.gimp_context_set_foreground("#000000")
    num_segs = 30
    segs = [20,16, 62,25, 70,30, 75,35, 78,42, 85,58, 98,76, 134,110, 134,93, 140,95, 145,96, 150,97, 150,197, 20,50, 20,16]
    pdb.gimp_image_select_polygon(img, 2, num_segs, segs)
    pdb.gimp_edit_bucket_fill(batsLayer, BUCKET_FILL_FG, LAYER_MODE_NORMAL, 100, 255, FALSE, 0, 0)
    pdb.gimp_selection_none(img)
    #Wings fingers
    #1
    pdb.gimp_brushes_set_brush("2. Hardness 100")
    pdb.gimp_context_set_brush_size(51)
    pdb.gimp_context_set_brush_aspect_ratio(0)
    pdb.gimp_context_set_brush_angle(0)
    pdb.gimp_context_set_brush_spacing(0.01)
    pdb.gimp_context_set_brush_default_hardness()
    pdb.gimp_context_set_brush_force(1)
    pdb.gimp_context_set_dynamics("Dynamics Off")
    pdb.gimp_context_set_paint_method("gimp-paintbrush")
    pdb.gimp_eraser_default(batsLayer, 2 , [4,35])
    #2
    pdb.gimp_context_set_brush_size(65)
    pdb.gimp_eraser_default(batsLayer, 2 , [33,79])
    #3
    pdb.gimp_context_set_brush_size(95)
    pdb.gimp_eraser_default(batsLayer, 2 , [70,139])
    #4
    pdb.gimp_context_set_brush_size(95)
    pdb.gimp_eraser_default(batsLayer, 2, [103,200])
    
    batsLayerc = pdb.gimp_layer_new_from_drawable(batsLayer, img)
    pdb.gimp_image_insert_layer(img, batsLayerc, None, 0)
    pdb.gimp_layer_set_mode(batsLayerc, LAYER_MODE_NORMAL_LEGACY)
    pdb.gimp_layer_set_opacity(batsLayerc, 100)
    batsLayerc = pdb.gimp_drawable_transform_flip_simple(batsLayerc, 
	                                         ORIENTATION_HORIZONTAL,   #flip_type 
															   TRUE,   #auto_center 
															      0,   #axis 
															   TRUE)   #clip_result
                                                               
    
    batsLayer = pdb.gimp_image_merge_down(img, batsLayerc, CLIP_TO_IMAGE)
    
    batsLayer2 = pdb.gimp_layer_new_from_drawable(batsLayer, img)
    pdb.gimp_image_insert_layer(img, batsLayer2, None, 0)
    pdb.gimp_layer_set_mode(batsLayer2, LAYER_MODE_NORMAL_LEGACY)
    pdb.gimp_layer_set_opacity(batsLayer2, 100)
    
    pdb.gimp_layer_scale(batsLayer, 200, 175, TRUE)
    pdb.gimp_layer_resize_to_image_size(batsLayer)    
    pdb.gimp_drawable_offset(batsLayer, FALSE, OFFSET_TRANSPARENT, 500, 70)
    batsLayer = pdb.gimp_item_transform_rotate(batsLayer, 0.54, TRUE, 700, 425)
    pdb.gimp_layer_resize_to_image_size(batsLayer)    
    
    pdb.gimp_layer_scale(batsLayer2, 130, 114, TRUE)
    pdb.gimp_layer_resize_to_image_size(batsLayer2)    
    pdb.gimp_drawable_offset(batsLayer2, FALSE, OFFSET_TRANSPARENT, 600, 0)
    batsLayer2 = pdb.gimp_item_transform_rotate(batsLayer2, -0.43, TRUE, 700, 425)
    pdb.gimp_layer_resize_to_image_size(batsLayer2)  

    #Create the bottom text		
    size = 400
    textW = size/0.3
    textH = size/3
    pdb.gimp_context_set_foreground(textColor)
    textLayer = pdb.gimp_text_fontname(img, None, 0, 0, customtext, 10, 1, size, PIXELS, fontname)
    pdb.gimp_text_layer_set_justification(textLayer, 2)
    pdb.gimp_text_layer_set_letter_spacing(textLayer, 20)    
    #Resize text
    pdb.gimp_layer_scale (textLayer, textW, textH, TRUE)

    #Making the text centered	
    textLayer = pdb.gimp_image_get_active_layer(img)
    pdb.gimp_image_set_active_layer(img, textLayer)
    wh = pdb.gimp_drawable_width(textLayer)
    hh = pdb.gimp_drawable_height(textLayer)
    w = pdb.gimp_image_width(img)
    h = pdb.gimp_image_height(img)
    pdb.gimp_image_set_active_layer(img,textLayer)
    centerx=w/2
    centery=h/0.99
    adw=wh/2
    adh=hh/0.7
    sw=centerx - adw
    sh=centery - adh		
    pdb.gimp_layer_set_offsets(textLayer, sw, sh)
    pdb.gimp_layer_resize_to_image_size(textLayer)

    #Text effect
    pdb.plug_in_gauss(img, textLayer, 10, 10, 1)
    pdb.plug_in_emboss(img, textLayer, 30, 45, 7, FALSE)
    pdb.plug_in_unsharp_mask(img, textLayer, 17, 3, 0)

    pdb.gimp_image_set_active_layer(img, backLayer)
    
    #Resets the previous user defaults
    pdb.gimp_context_pop()	
    
register(
    "pm_halloween_graveyard_scene",
    "Creates a halloween scene",
    "Creates a halloween graveyard scene",
    "Pocholo",
    "Pocholo",
    "2022",
    "Halloween Graveyard scene",
    "",
    [
    (PF_STRING, "customtext", "Text string", 'TRICK OR TREAT'),
    (PF_FONT, "fontname", "Font name", "CF Halloween"),
    (PF_COLOR, "hazeglowColor", "Haze & Color", (255, 80, 0)),
    (PF_COLOR, "textColor", "Text Color", (255, 80, 0)),   
    ],
    [],
    pm_halloween_graveyard_scene, menu="<Image>/Python-Fu/Halloween",
    domain=("gimp20", gimp.locale_directory))
main()	                       